#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Sample indicator demonstrating the use of the DrawRegion() method.
    /// </summary>
    [Description("Sample indicator demonstrating the use of the DrawRegion() method.")]
    public class SampleDrawRegion : Indicator
    {
        #region Variables
		private	double		numStdDev	= 2;
		private int			period		= 14;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			// Setting Overlay to true means our indicator will be defaulted to plot on the price panel
			Overlay				= true;
            CalculateOnBarClose	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// With the DrawRegion() method you can fill the space inbetween two DataSeries objects very easily.
			
			/* This region fills in the space between the Upper Bollinger Band and the Middle Bollinger Band. The region extends from the first bar of the chart
			till the last bar of the chart. It has a border color of black and is filled with blue on an opacity setting of 2. Opacity setting ranges from 0-10.
			0 being transparent and 10 being completely colored. */
			DrawRegion("Bollinger Upper Region", CurrentBar, 0, Bollinger(NumStdDev, Period).Upper, Bollinger(NumStdDev, Period).Middle, Color.Black, Color.Blue, 2);
			
			/* This region fills the space between the Lower Bollinger Band and the Middle Bollinger Band. It has the same attributes as the previous region, except
			the blue fill color is darker in this one. If you wish to create a region without a border you can use this color: Color.Transparent */
			DrawRegion("Bollinger Lower Region", CurrentBar, 0, Bollinger(NumStdDev, Period).Lower, Bollinger(NumStdDev, Period).Middle, Color.Black, Color.Blue, 5);
			
			/* Besides filling inbetween two DataSeries objects we can also fill between a double value and a DataSeries.
			This is demonstrated in the following code segment. */
			
			// If the price closes above the upper bollinger band, color the price region above the bollinger band gold.
			if (Bollinger(NumStdDev, Period).Upper[0] < Close[0])
			{
				/* In our string tag we use "+ CurrentBar" to ensure unique tag names for all our regions. If we did not have unique names each call
				upon the tag would modify the existing DrawRegion() instead of coloring a new one. */
				DrawRegion("Upper Bollinger Broken" + CurrentBar, 1, 0, Bollinger(NumStdDev, Period).Upper, High[0], Color.Black, Color.Gold, 10);
			}
			
			// If the price closes below the lower bollinger band, color the price region below the bollinger band gold.
			else if (Bollinger(NumStdDev, Period).Lower[0] > Close[0])
			{
				DrawRegion("Lower Bollinger Broken" + CurrentBar, 1, 0, Bollinger(NumStdDev, Period).Lower, Low[0], Color.Black, Color.Gold, 10);
			}
        }

        #region Properties
		/// <summary>
		/// </summary>
		[Description("Number of standard deviations")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("# of std. dev.")]
		public double NumStdDev
		{
			get { return numStdDev; }
			set { numStdDev = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleDrawRegion[] cacheSampleDrawRegion = null;

        private static SampleDrawRegion checkSampleDrawRegion = new SampleDrawRegion();

        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        public SampleDrawRegion SampleDrawRegion(double numStdDev, int period)
        {
            return SampleDrawRegion(Input, numStdDev, period);
        }

        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        public SampleDrawRegion SampleDrawRegion(Data.IDataSeries input, double numStdDev, int period)
        {
            checkSampleDrawRegion.NumStdDev = numStdDev;
            numStdDev = checkSampleDrawRegion.NumStdDev;
            checkSampleDrawRegion.Period = period;
            period = checkSampleDrawRegion.Period;

            if (cacheSampleDrawRegion != null)
                for (int idx = 0; idx < cacheSampleDrawRegion.Length; idx++)
                    if (Math.Abs(cacheSampleDrawRegion[idx].NumStdDev - numStdDev) <= double.Epsilon && cacheSampleDrawRegion[idx].Period == period && cacheSampleDrawRegion[idx].EqualsInput(input))
                        return cacheSampleDrawRegion[idx];

            SampleDrawRegion indicator = new SampleDrawRegion();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.NumStdDev = numStdDev;
            indicator.Period = period;

            SampleDrawRegion[] tmp = new SampleDrawRegion[cacheSampleDrawRegion == null ? 1 : cacheSampleDrawRegion.Length + 1];
            if (cacheSampleDrawRegion != null)
                cacheSampleDrawRegion.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSampleDrawRegion = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleDrawRegion SampleDrawRegion(double numStdDev, int period)
        {
            return _indicator.SampleDrawRegion(Input, numStdDev, period);
        }

        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleDrawRegion SampleDrawRegion(Data.IDataSeries input, double numStdDev, int period)
        {
            return _indicator.SampleDrawRegion(input, numStdDev, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleDrawRegion SampleDrawRegion(double numStdDev, int period)
        {
            return _indicator.SampleDrawRegion(Input, numStdDev, period);
        }

        /// <summary>
        /// Sample indicator demonstrating the use of the DrawRegion() method.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleDrawRegion SampleDrawRegion(Data.IDataSeries input, double numStdDev, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleDrawRegion(input, numStdDev, period);
        }

    }
}
#endregion
